#!/usr/bin/php
<?php

/* Change this variable if you've changed the mysql root password (this script will create a new database). */
$root_password = 'nagiosxi';


/*
This script heavily utilizes code from the 'Pg2MySQL' converter project
http://www.lightbox.org/pg2mysql.php

Copyright (C) 2005-2011 James Grant <james@lightbox.org>
            Lightbox Technologies Inc.

This script is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/


require_once(dirname(__FILE__).'/../../html/config.inc.php');
include "pg2mysql.inc.php";

function shell_command_or_die($cmd)
{
    $output = array();
    $rc = -1;
    exec($cmd, $output, $rc);
    if ($rc != 0) {
        $step_name = explode($cmd, " ");
        echo "$step_name failed!\n";
        foreach ($output as $line) {
            echo "$line";
        }
        exit;
    }
}

$nagiosxi_dbtype = $cfg['db_info']['nagiosxi']['dbtype'];
if ($nagiosxi_dbtype != "pgsql") {
    echo "Your config.inc.php says you're using '$nagiosxi_dbtype', not 'pgsql'. You must be using pgsql to migrate away from it.";
    exit;
}

$username = $cfg['db_info']['nagiosxi']['user'];
$password = $cfg['db_info']['nagiosxi']['pwd'];
$database = $cfg['db_info']['nagiosxi']['db'];
$base_directory = dirname(__FILE__);
$input_path = $base_directory . "/nagiosxi.psql";
$output_path = $base_directory . "/nagiosxi.mysql";

echo "Dumping to nagiosxi.psql...\n";
$cmd = "pg_dump --no-acl --no-owner --format p --data-only -U ".escapeshellarg($username)." " . escapeshellarg($database) . " >$input_path";
shell_command_or_die($cmd);
echo "Done!\n";

echo "Creating $output_path...\n";
$config['engine']="InnoDB";
pg2mysql_large($input_path, $output_path);
echo "Done!\n";

echo "Creating Nagios XI database...\n";
$create_db_path = dirname(__FILE__)."/nagiosxi_create_db.sql";
$cmd = "mysql -uroot -p$root_password < $create_db_path";
shell_command_or_die($cmd);
echo "Done!\n";

echo "Importing PostgreSQL data to MySQL...\n";
$cmd = "mysql -uroot -p$root_password --force $database < $output_path";
shell_command_or_die($cmd);
echo "Done!\n";

echo "Creating '$username' user in MySQL...\n";
$cmd = "mysql -u root -p$root_password -e \"CREATE USER '$username'@'localhost' IDENTIFIED BY '$password';\"";
shell_command_or_die($cmd);
$cmd = "mysql -u root -p$root_password -e \"GRANT ALL ON $database.* TO '$username'@'localhost'; FLUSH PRIVILEGES;\"";
shell_command_or_die($cmd);
echo "Done!\n";

echo "Modifying config.inc.php to use mysql...\n";
$cmd = 'cp -p /usr/local/nagiosxi/html/config.inc.php /usr/local/nagiosxi/html/config.inc.php.backup';
shell_command_or_die($cmd);
$cmd = "sed -i 's/pgsql/mysql/' /usr/local/nagiosxi/html/config.inc.php";
shell_command_or_die($cmd);

// We don't care if this one succeeds.
$cmd = "mv /usr/local/nagiosxi/html/includes/components/nagiosim ~/nagiosim.backup &>/dev/null";
exec($cmd);

echo "Done!\n";

echo "Disabling PostgreSQL, restarting MySQL...";
$cmd = 'service mysqld restart || systemctl restart mariadb.service';
shell_command_or_die($cmd);
$cmd = "service postgresql stop || systemctl stop postgresql.service";
shell_command_or_die($cmd);
$cmd = "chkconfig postgresql off || systemctl disable postgresql.service";
shell_command_or_die($cmd);
echo "Done!\n";

echo <<<XHTML

========
FINISHED
========

All of our migration commands succeeded without any issues. 
PostgreSQL is disabled, and MySQL is enabled.
Please try to log into your Nagios XI interface and confirm that it's working.
If you succeed, please reboot this server and verify that the web interface is working after.

If you run into issues, you will need to run the following commands:

    mv /usr/local/nagiosxi/html/config.inc.php.backup /usr/local/nagiosxi/html/config.inc.php
    chkconfig postgresql on || systemctl enable postgresql
    service postgresql start || systemctl start postgresql

If you did roll back, you'll also need to get into contact with the Nagios XI Support team. 
In your ticket, please include a zip of this directory:

$base_directory

XHTML;

